'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _commandExists = require('command-exists');

var _commandExists2 = _interopRequireDefault(_commandExists);

var _os = require('os');

var _os2 = _interopRequireDefault(_os);

var _ProcessRunner = require('./ProcessRunner');

var _ProcessRunner2 = _interopRequireDefault(_ProcessRunner);

var _ora = require('ora');

var _ora2 = _interopRequireDefault(_ora);

var _inquirer = require('inquirer');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const ensure = args => {
  if (_os2.default.platform() !== 'darwin') {
    return Promise.reject('Only macOS is supported.');
  }

  if (_commandExists2.default.sync('kd')) {
    return Promise.resolve();
  }

  if (_commandExists2.default.sync('brew')) {
    const runner = new _ProcessRunner2.default({
      name: 'brew',
      dir: process.cwd()
    });

    const spinner = (0, _ora2.default)('Running `brew tap rjeczalik/kd`');
    return Promise.resolve().then(() => showWelcomeMessage()).then(() => askForPermission()).then(() => spinner.start()).then(() => runner.run(['tap', 'rjeczalik/kd'])).then(() => spinner.text = 'Running `brew install kd --devel`').then(() => runner.run(['install', 'kd', '--devel'])).then(() => runner.run(['link', 'kd'])).then(() => spinner.succeed('kd installed successfully')).then(() => console.log(`running kd ${args.join(' ')}`)).then(() => Promise.resolve(args));
  }

  return Promise.reject('You need to install Homebrew first.');
};

const showWelcomeMessage = () => {
  console.log(`

    Welcome to KDX!

  `);
};

const askForPermission = () => {
  return (0, _inquirer.prompt)([{
    type: 'confirm',
    name: 'allowed',
    message: '`kd` is missing in your system. Do you want to install?' }]).then(({ allowed }) => allowed ? Promise.resolve() : Promise.reject('Aborted'));
};

exports.default = ensure;